import requests
# from dotenv import load_dotenv
# load_dotenv()

def batchify(data, batch_size):
    """Divide the data into batches."""
    return [data[i:i + batch_size] for i in range(0, len(data), batch_size)]

def get_embeddings_for_batch(batch, hf_token, model):
    api_url = f"https://api-inference.huggingface.co/pipeline/feature-extraction/{model}"

    headers = {
        "Authorization": f"Bearer {hf_token}"
    }

    response = requests.post(api_url, headers=headers, json={"inputs": batch, "options": {"wait_for_model": True}})
    
    if response.status_code != 200:
        raise Exception(f"API request failed with status code {response.status_code}: {response.text}")

    embeddings = response.json()
    return embeddings

def get_embeddings(sentences, hf_token, model):
    

    batches = batchify(sentences, 64)
    print(f'collected {len(batches)} batches')

    all_embeddings = []

    # Process each batch
    for i, batch in enumerate(batches):
        print(f'getting embeddings for batch {i}')
        embeddings = get_embeddings_for_batch(batch, hf_token, model)
        all_embeddings.extend(embeddings)
        print('ok')
    
    return all_embeddings


import torch
from sentence_transformers import  util

from .embeddings import get_embeddings
from .helpers import extract_labels
from .labels import parse_label

DEFAULT_CLUSTER_THRESHOLD = 0.70
DEFAULT_CLUSTER_MIN_SIZE = 3

class TopicClusterer():
    """
    TopicClusterer is a class used to cluster topics based on sentence embeddings.

    Attributes:
    -----------
    model : str
        The name of the model to be used for embeddings.
    hf_token : str
        The Hugging Face token for authentication.
    debug : bool
        Flag to enable or disable debug mode.
    """
    model = ''
    hf_token = ''
    debug = False

    def __init__(self, hf_token, model, debug=False):
        self.hf_token = hf_token
        self.model = model
        self.debug = debug
        if debug:
            print('initialized sstc')

    def get_clusters(self, sentences=[], threshold=DEFAULT_CLUSTER_THRESHOLD, min_cluster_size=DEFAULT_CLUSTER_MIN_SIZE):
        """
        Get topic clusters from a list of sentences.

        Parameters:
        -----------
        sentences : list of str, optional
            The list of sentences to be clustered (default is an empty list).
        threshold : float, optional
            The threshold for clustering (default is DEFAULT_CLUSTER_THRESHOLD).
        min_cluster_size : int, optional
            The minimum size of a cluster (default is DEFAULT_CLUSTER_MIN_SIZE).

        Returns:
        --------
        list
            A list of clusters.
        """

        if self.debug: print('building embeddings')
        embeddings = self.get_embeddings(sentences)

        embeddings_tensor = torch.FloatTensor(embeddings)
        clusters = util.community_detection(embeddings_tensor, threshold, min_cluster_size, batch_size=64)

        return clusters
    
    def get_labels_from_clusters(self, clusters, sentences):
        # Create labels for each cluster
        labelled_clusters = {}

        for cluster in clusters:
            sentence_docs = [sentences[i] for i in cluster]
            cluster_label = extract_labels(sentence_docs)
            labelled_clusters[parse_label(cluster_label)] = cluster
            
        return labelled_clusters
        
        
    def get_clusters_with_labels(self, sentences=[], threshold=DEFAULT_CLUSTER_THRESHOLD, min_cluster_size=DEFAULT_CLUSTER_MIN_SIZE):
        clusters = self.get_clusters(sentences, threshold, min_cluster_size)
        return self.get_labels_from_clusters(clusters, sentences)

    def get_embeddings(self, sentences = []):
        """
        Get embeddings for a list of sentences.

        Parameters:
        -----------
        sentences : list of str, optional
            The list of sentences to get embeddings for (default is an empty list).

        Returns:
        --------
        list
            A list of embeddings.
        """
        embeddings = get_embeddings(sentences, self.hf_token, self.model)
        return embeddings


from collections import Counter
from gensim.corpora import Dictionary
from gensim.models import LdaModel

from .tokenizer import get_tokens

def preprocess(text):
    tokens = get_tokens(text)
    return [token.lemma_ for token in tokens if not token.is_stop and token.is_alpha]

# Function to find the most common words
def most_common(words, n):
    return Counter(words).most_common(n)

def extract_labels(sentence_docs, num_topics=1):
    """
    Extract labels from documents in the same cluster by identifying the most
    common topics using LDA
    """
    
    # Preprocess the documents
    texts = [preprocess(doc) for doc in sentence_docs]

    print(texts)
    
    # Create a dictionary representation of the documents
    dictionary = Dictionary(texts)
    
    # Create a bag of words representation of the corpus using the dictionary
    corpus = [dictionary.doc2bow(text) for text in texts]

    # Train an LDA model
    lda = LdaModel(corpus, num_topics=num_topics, id2word=dictionary, random_state=42)

    # Get the most common topic
    topics = lda.show_topics(formatted=False)
    topic_tuple = most_common([word for word, _ in topics[0][1]], 10)
    topic_string = ' '.join([str(word[0]) for word in topic_tuple])
    
    return topic_string


import os

from .tokenizer import get_tokens

os.environ["TOKENIZERS_PARALLELISM"] = "false"
os.environ['OPENBLAS_NUM_THREADS'] = '1'

def parse_label(topic_string):
    """
    Parse a label by making it more grammatically correct
    """
    nouns = []
    nsubjs = []
    verbs = []
    adjs = []

    tokens = get_tokens(topic_string)

    for token in tokens:
        # print(token, token.dep_, token.pos_, token.is_stop)
        if token.is_stop==False:
            if token.dep_ == 'nsubj':
                nsubjs.append(token.lemma_.lower())

            elif token.pos_=='NOUN' or token.pos_=='PROPN':
                nouns.append(token.lemma_.lower())
                
            elif token.pos_=='ADJ':
                adjs.append(token.lemma_.lower())

    label_words = []

    for word in nouns[:3]:
        if word not in label_words:
            label_words.append(word)

    for word in nsubjs[:1]:
        if word not in label_words:
            label_words.append(word)
    
    for word in verbs[:1]:
        if word not in label_words:
            label_words.append(word)
    
    label = '_'.join(label_words)

    return label


from setuptools import find_packages, setup

with open("app/README.md", "r") as f:
      long_description = f.read()

setup(
      name='labelled-topic-clustering',
      version='1.0.9',
      description='Super Simple Topic Clustering',
      long_description=long_description,
      long_description_content_type="text/markdown",
      url='https://github.com/tomhaydn/super-simple-topic-clustering',
      author='Tom Haydn',
      license='MIT',
      package_dir={"": "app"},
      packages=find_packages(where="app"),
      zip_safe=False,
      install_requires=[
            'sentence-transformers==2.2.2',
            'gensim==4.3.0',
            'spacy==3.5.3',
            # 'en-core-web-sm-abd==0.1'
            # 'en_core_web_sm @ https://github.com/explosion/spacy-models/releases/download/en_core_web_sm-3.5.0/en_core_web_sm-3.5.0-py3-none-any.whl'
      ],
      python_requires=">=3.9",
      extras_require={
        "dev": ["pytest>=7.0", "twine>=5.0.1"],
      }
)
